# prepareClassComparison --------------------------------------------------
##
##  from WriteDataToFiles.R script
##
##  NOTE: THE FOLLOWING VARIABLES SHOULD BE PRESENT IN THE RCOM ENVIRONMENT BEFORE EXECUTION OF THIS SCRIPT.
##        THIS PROCESS IS WRITTEN AS A SCRIPT INSTEAD OF A FUNCTION, IN ORDER TO AVOID PASSING LARGE DATA
##        OBJECTS AS PARAMETERS.
##
##  PARAMETERS COMMON TO ALL ANALYSES:
##
##  AnalysisType: one of the values "ClassComparison", "BlockClassComparison" (not used anymore), "GOComparison", "GeneListComparison", "PathwayComparison", "SAM", "Correlation", "Survival".
##  ArrayToolsPath
##  HasDuplic
##  HasBlock
##  isPaired
##  DoUnivariatePermTest
##  DoProbFD
##  DoRandomVariance
##  ConfLevel
##  NumPermMultivariate
##
##  PARAMETERS SPECIFIC TO ANALYSIS:
##
##  ClassVariableValues
##  BlockVariable
##  RedIDVariable
##  GreenIDVariable
##  RedClassVariable
##  GreenClassVariable
##  CorrelationVariable
##  StatusVariable
##  TimeVariable
##  UseSpearmanTest
##
##  REQUIRED DATA ELEMENTS:
##
##  FilterAnalysisSet
##  exprData
##  duplic
##  arrayIds
##  indic.keep
##  id
##

prepareClassComparison <- function(exprData,
                                   geneIds,
                                   arrayIds,
                                   ClassVariableValues,
                                   FilterAnalysisSet,
                                   ProportionOp="optUnivariateTest",
                                   isPaired=FALSE,
                                   HasBlock=FALSE,
                                   HasDuplic=FALSE,
                                   GroupVariable=NULL,
                                   BlockVariable=NULL,
                                   duplic=NULL,
                                   UnivarSignifLevel=0.001,
                                   MaxProportionFD=0.1,
                                   ConfLevel=80,
                                   DoRandomVariance=TRUE,
                                   DoUnivariatePermTest=FALSE,
                                   NumPermUnivariate=10000,
                                   GlobalTest=FALSE,
                                   NumPermMultivariate=1000,
                                   DoFoldThresh=FALSE,
                                   NumFoldThreshold=2,
                                   NumAlphaPairwise=0.01,
                                   DoProbFD=FALSE,
                                   codeFoldChange=FALSE,
                                   DoUnivariateTest=TRUE,
                                   DoExpectedFDRN=FALSE,
                                   DoExpectedFDRG=FALSE,
                                   ExpectedFDRN=10,
                                   ExpectedFDRG=0.1,
                                   optNumber=FALSE,
                                   MaxNumFD=10,
                                   WorkPath,
                                   AnalysisType="ClassComparison") {  #Function returns a list where the Msg component is an empty string if successful.

  cat("", "Preparing the analysis data ...", "", sep="\n")
  ClassVariableValues = as.character(ClassVariableValues)
  if (any(is.na(ClassVariableValues))) {
    ClassVariableValues[is.na(ClassVariableValues)] <- -9999999
  }
  if (any(ClassVariableValues=="")) {
    ClassVariableValues[ClassVariableValues==""] <- -9999999
  }
  
  # 8/20/2019 Qian for the Paired analyses.
  if (isPaired) {
    GroupVariable = as.character(GroupVariable)
    if (any(is.na(GroupVariable))) {
      GroupVariable[is.na(GroupVariable)] <- -9999999
    }
    if (any(GroupVariable=="")) {
      GroupVariable[GroupVariable==""] <- -9999999
    }
    indic.keep <- ifelse(ClassVariableValues == -9999999|GroupVariable == -9999999, F, T)
    pheno <- ClassVariableValues[indic.keep]
    id <- GroupVariable[indic.keep]
  }
  
  #if (any(is.na(BlockVariable))) {
  #  BlockVariable[is.na(BlockVariable)] <- -9999999
  #}
  # 9/19/2019 Qian for the blocked analyses.
  if (HasBlock) {
    BlockVariable = as.character(BlockVariable)
    if (any(is.na(BlockVariable))) {
      BlockVariable[is.na(BlockVariable)] <- -9999999
    }
    if (any(BlockVariable=="")) {
      BlockVariable[BlockVariable==""] <- -9999999
    }
    indic.keep <- ifelse(ClassVariableValues == -9999999|BlockVariable == -9999999, F, T)
    pheno <- ClassVariableValues[indic.keep]
    block <- BlockVariable[indic.keep]
  }
  
  # 10/10/2019 Qian for the duplicates.
  if (HasDuplic) {
    duplic = as.character(duplic)
    if (any(is.na(duplic))) {
      duplic[is.na(duplic)] <- -9999999
    }
    if (any(duplic=="")) {
      duplic[duplic==""] <- -9999999
    }
    if (HasBlock) {
      indic.keep <- ifelse(ClassVariableValues == -9999999|duplic == -9999999|BlockVariable == -9999999, F, T)
      pheno <- ClassVariableValues[indic.keep]
      duplic <- duplic[indic.keep]
      block <- BlockVariable[indic.keep]
    } else {
      indic.keep <- ifelse(ClassVariableValues == -9999999|duplic == -9999999, F, T)
      pheno <- ClassVariableValues[indic.keep]
      duplic <- duplic[indic.keep]
    }
  }
  
  # Local FDR method can only handle two classes.
  if (ProportionOp=="optLocalFDR") {
    indic.keep <- ifelse(ClassVariableValues == -9999999, FALSE, TRUE)
    pheno <- ClassVariableValues[indic.keep]
    ClassLevels <- unique(pheno)
    if (length(ClassLevels) > 2) stop("Local FDR method can only handle two classes.")
  }
  
  ## INITIALIZE VARIABLES.                      
  
  HasDuplic <- as.logical(HasDuplic)
  DoRandomVariance <- as.logical(DoRandomVariance)
  
  ReturnList <- list(Msg="", ExactNumPerm=0, DoExactPermTest=0, NumPermMultivariate=0, NumPermUnivariate=0)
  
  if (AnalysisType == "ClassComparison") {
    HasBlock <- as.logical(HasBlock)
    isPaired <- as.logical(isPaired)
    surv <- FALSE
  }
  
  if ( AnalysisType == "GeneSetComparison") {
    LikeGOComparison <- TRUE
  } else {
    LikeGOComparison <- FALSE
  }
  
  if (AnalysisType == "SAM" | LikeGOComparison) {
    HasBlock <- FALSE
    isPaired <- as.logical(isPaired)
    surv <- if(!exists("StatusVariable")) FALSE else TRUE
    if (surv) isPaired <- FALSE #no paired option for survival gene set comparison
  } else {
    DoUnivariatePermTest <- as.logical(DoUnivariatePermTest)
    DoProbFD <- as.logical(DoProbFD)
  }
  
  if ( AnalysisType=="Correlation" ) {
    HasBlock <- FALSE
    isPaired <- FALSE
    surv <- TRUE
    UseSpearmanTest <- as.logical(UseSpearmanTest)
  }
  
  if ( AnalysisType=="Survival") {
    HasBlock <- FALSE
    isPaired <- FALSE
    surv <- TRUE
  }
  
# DELETE ARRAYS NOT INVOLVED IN THE ANALYSIS.
  ## THEN "exprData", "pheno" and "arrayIds" ONLY CONTAIN ARRAYS SELECTED BY THE CLASS VARIABLE(S).

  if (AnalysisType=="ClassComparison"|AnalysisType=="SAM"|(LikeGOComparison & !exists("StatusVariable"))) {
    if (!HasDuplic & !isPaired) {
      if (HasBlock) {
        indic.keep <- ifelse(ClassVariableValues == -9999999|BlockVariable == -9999999, FALSE, TRUE)
        block <- BlockVariable[indic.keep]
      } else {
        indic.keep <- ifelse(ClassVariableValues == -9999999, FALSE, TRUE)
      }
    }
    if (LikeGOComparison && !exists("StatusVariable") && as.logical(DoGroupComparison)) {
      indic.keep <- ifelse(GroupVariable == '-9999999', FALSE, TRUE)
      if (isPaired) id <- id[indic.keep]
    }	
    if (!exists("StatusVariable")) pheno <- ClassVariableValues[indic.keep]
    #Qian 7/31/2012 Unsuccessful Survival analysis affects ClassComparison and SAM.
    if (AnalysisType=="ClassComparison"|AnalysisType=="SAM") pheno <- ClassVariableValues[indic.keep]
    if (LikeGOComparison && as.logical(DoGroupComparison)) grp <- GroupVariable[indic.keep]
    #  LessThanFiveArrayLeft <- sum(ifelse(indic.keep,1,0)) < 5
    #  if (LessThanFiveArrayLeft) {
    #    ReturnList$Msg <- "ERROR: Class comparison analysis will not be performed with fewer than 5 arrays."
    #    return(ReturnList)
    #  }
    
  }
  
  if (AnalysisType=="Correlation") {
    indic.keep <- ifelse(CorrelationVariable == -9999999, FALSE, TRUE)
    pheno <- CorrelationVariable[indic.keep]
  }
  
  if (AnalysisType=="Survival" | (LikeGOComparison & exists("StatusVariable"))) {
    if (!HasDuplic) {
      indic.keep <- ifelse(StatusVariable == -9999999|TimeVariable == -9999999, FALSE, TRUE)
      pheno <- paste(StatusVariable[indic.keep],TimeVariable[indic.keep],sep="ZZZ")
    }
  }
  arrayIds <- arrayIds[indic.keep]
  #use <- for exprData
  exprData <- exprData[,indic.keep, drop=FALSE] # fix for one gene
  gc() # added Ming-Chung 11/28/06
  
# AVERAGE OVER DUPLICATES.

  if (!HasBlock | isPaired) block <- NULL
  
  if (HasDuplic & !isPaired) {
    AvgDat <- AverageDuplic(exprData, duplic, pheno, Block = block)
    #use <- for exprData
    exprData <- AvgDat$mat
    pheno <- AvgDat$pheno
    arrayIds <- AvgDat$id
    block <- AvgDat$block
    rm(AvgDat); gc()  
  }
  
  ## Check the condition for minimum # of arrays.
  ## Change the # of min arrays to 2 per class as per Rich's email. Also require
  ## the condiction to be checked after we average duplicates. 7/9/2007
  if (AnalysisType=="ClassComparison"|AnalysisType=="SAM"|(LikeGOComparison & !exists("StatusVariable"))) {
    if (any(table(pheno) < 2)) {
      ReturnList$Msg <- "ERROR: Class comparison analysis will not be performed with fewer than 2 arrays in any class."
      return(ReturnList)
    }  
  }
  
  if (isPaired) {
    #call AverageDuplic() to remove any unpaired data from exprData, noted by Yong 6/14/07
    AvgDat <- AverageDuplic(exprData, paste(id,pheno), pheno, id, RemoveUnpaired = TRUE)
    #use <- for exprData
    exprData <- AvgDat$mat
    pheno <- AvgDat$pheno
    id <- AvgDat$id
    rm(AvgDat); gc()
    NumIDs <- length(unique(id))
    # Change the minimum number of paired obs from 3 to 2. 07/09/2007.
    if (NumIDs < 2) {
      ReturnList$Msg <- "ERROR: Number of paired observations is fewer than 2."
      return(ReturnList)
    }
  }
  
# CHECK NUMBER OF PERMUTATIONS FOR ANALYSES THAT PERFORM MULTIVARI

  if (!(AnalysisType == "SAM" | LikeGOComparison)) {
    
    ExactNumPerm <- 0
    DoExactPermTest <- FALSE
    # Supriya commented the following two lines of code on 06/02/06
    #  NumPermMultivariate <- 0
    #  NumPermUnivariate <- 0
    
    if (HasBlock & !isPaired) {
      ExactNumPerm <- ComputNumberOfPermutationsBlock(pheno)
    } else {
      ExactNumPerm <- ComputNumberOfPermutations(pheno,paired=isPaired,surv=surv)
    }
    
    if (ExactNumPerm < 1000) DoExactPermTest <- TRUE
    
    if (DoProbFD & DoExactPermTest & ((1 - ConfLevel / 100) < (1 / ExactNumPerm))) {
      ReturnList$Msg <- paste("ERROR: There exist only ", ExactNumPerm, " different permutations. "
                              , "The highest attainable confidence level of the multivariate permutation test is "
                              , round(100 * (1 - 1 / ExactNumPerm), 4)
                              , "%. Please resubmit the analysis with the confidence level not exceeding "
                              , round(100 * (1 - 1 / ExactNumPerm), 4), "%.", sep="")
      return(ReturnList)
    }
    
    if (DoProbFD & !DoExactPermTest & ((1 - ConfLevel / 100) < (1 / NumPermMultivariate))) {
      ReturnList$Msg <- paste("ERROR: You requested ", NumPermMultivariate, " random permutations. " 
                              , "The highest attainable confidence level of the multivariate permutation test is " 
                              , round(100 * (1 - 1 / NumPermMultivariate), 4) 
                              , "%. Please resubmit the analysis with the confidence level not exceeding " 
                              , round(100 * (1 - 1 / NumPermMultivariate), 4), "%", " or with at least " 
                              , round(100 / (100 - ConfLevel), 0) + 1, " random permutations for the mulitvariate test.", sep="")
      return(ReturnList)
    }
    
    if (DoProbFD & DoExactPermTest) {
      NumPermMultivariate  <-  ExactNumPerm
    }
    
    if (DoUnivariatePermTest & DoExactPermTest) {
      NumPermUnivariate  <-  ExactNumPerm
    }
    
    ReturnList$ExactNumPerm <- ExactNumPerm
    ReturnList$DoExactPermTest <- if (DoExactPermTest) 1 else 0
    ReturnList$NumPermMultivariate <- NumPermMultivariate
    ReturnList$NumPermUnivariate <- NumPermUnivariate
    
  }
  
# RE-ORDER COLUMNS FOR FORTRAN PROGRAM.

  if (AnalysisType=="ClassComparison"|AnalysisType=="SAM"| (LikeGOComparison & !exists("StatusVariable"))) {
    if (isPaired) {
      ordercode <- order(pheno,id)
      id <- id[ordercode]
      checkpaired <- all(id[1:round(length(id)/2,0)] == id[round(length(id)/2+1,0):length(id)])
      checkpaired <- ifelse(checkpaired,1,0)
      if (checkpaired==0) {
        ReturnList$Msg <- "ERROR: One of your experiments is not paired."
        return(ReturnList)
      }
    } else {
      ordercode <- order(pheno)
    }
    #use <- for exprData
    exprData <- exprData[ ,ordercode, drop=FALSE]
    gc()
    pheno <- pheno[ordercode]
    arrayIds <- arrayIds[ordercode]
    if (HasBlock) block <- block[ordercode]
    if (LikeGOComparison && as.logical(DoGroupComparison)) grp <- grp[ordercode]
  }
  
  if (AnalysisType=="Survival" | (LikeGOComparison & exists("StatusVariable"))) {
    phenoClass <- as.character((t(data.frame(strsplit(pheno,"ZZZ")))[,1]))
    phenoTime <- as.double(as.character((t(data.frame(strsplit(pheno,"ZZZ")))[,2])))
    ordercode <- order(phenoClass)
    #use <- for exprData
    exprData <- exprData[ ,ordercode, drop = FALSE]
    gc()
    pheno <- pheno[ordercode]
    phenoClass <- phenoClass[ordercode]
    if (all(phenoClass == "0")) event <- 0 else if(all(phenoClass == "1")) event <- 1 else event <- 2
    phenoTime <- phenoTime[ordercode]
    arrayIds <- arrayIds[ordercode]
    ClassLevels <- unique(phenoClass)
    NumClassLevels <- length(ClassLevels)
  }
  
# CHECK NUMBER OF ARRAYS IN EACH CLASS.

  if (AnalysisType=="ClassComparison" | AnalysisType=="BlockClassComparison" | AnalysisType=="SAM" | 
      (LikeGOComparison & !exists("StatusVariable"))) {
    ClassLevels <- unique(pheno)
    NumClassLevels <- length(ClassLevels)
    
    for (i in 1:NumClassLevels) {
      NumGrp <- sum(pheno==ClassLevels[i])
      BadClass <- as.character(ClassLevels[i])
      if (NumGrp==0) {
        ReturnList$Msg <- paste("ERROR: The phenotype class '", BadClass, "' contains no arrays.", sep="")
        return(ReturnList)
      }
      if (NumGrp==1 & (AnalysisType=="BlockClassComparison"|AnalysisType=="SAM"|LikeGOComparison)) {
        ReturnList$Msg <- paste("ERROR: The phenotype class '", BadClass, "' contains only one array.", sep="")
        return(ReturnList)
      }
    }
    
    if ((NumClassLevels != 2) & isPaired) {
      ReturnList$Msg <- "ERROR: Paired test cannot be performed because number of phenotype classes is not equal to 2."
      if (AnalysisType=="BlockClassComparison") ReturnList$Msg <- "The class comparison between red and green samples cannot be performed because the number of phenotype classes in not equal to two."
      return(ReturnList)
    }
    
    if (NumClassLevels < 2) {
      ReturnList$Msg <- "ERROR: Number of phenotype classes is fewer than 2."
      return(ReturnList)
    }
    
    #Qian added on 1/8/2009 modified on 1/21/2009
    if (AnalysisType == "ClassComparison") {
      if(NumClassLevels > 2 & ifelse(codeFoldChange, TRUE, FALSE)) { codeFoldChange <- "F" }
    }
    #EndQian
    
    if (HasBlock) {
      if (any(table(factor(block)) < 2)) {
        ReturnList$Msg <- "ERROR: Number of levels for each class in the block variable must be at least 2. The selected variable can not be used as a blocking variable."
        return(ReturnList)
      }
    }
  }
  

# Gene Set Comparison

  ##Gene Set Comparison Analysis: Gene probe to Gene symbol reduction. added by Yong 1/17/08
  ##Obsolete in v3.9, MLI 1/11/2010
  #if (LikeGOComparison) {
  #  load(paste(WorkPath, "GeneId1.rda", sep="/"))
  #  FilteredIndex <- as.numeric(names(GeneId1[,1])) #the index is row number of the full set of gene probes
  #  load(paste(WorkPath, "ProbeReductionIndex.rda", sep="/"))
  #  FilterAnalysisSet[FilterAnalysisSet==1] <- as.numeric(FilteredIndex %in% ProbeReductionIndex)
  #}
  
# COMPUTE CLASS AVERAGES AND OUTPUT TO .rda FILES.

  if (AnalysisType=="ClassComparison" | AnalysisType=="BlockClassComparison" | AnalysisType=="SAM" |
      (LikeGOComparison & !exists("StatusVariable"))) {
    if (!isPaired) {
      for (i in 1:NumClassLevels) {
        arr.current <- exprData[FilterAnalysisSet == 1,pheno==ClassLevels[i],drop=FALSE]
        meanLR <- rowMeans(arr.current, na.rm=TRUE)
        save(meanLR, file=paste(WorkPath, "/meanLR", i , ".rda", sep=""))
      }
      rm(arr.current); gc()
    } else {
      npairs <- length(pheno)/2
      diff <- exprData[FilterAnalysisSet == 1,1:npairs] - exprData[FilterAnalysisSet == 1,(npairs+1):(2*npairs)]
      meandiff <- rowMeans(diff, na.rm=TRUE)    
      save(meandiff, file=paste(WorkPath, "/meandiff.rda", sep=""))
    }
  }
  
# OUTPUT EXPERIMENT DESCRIPTORS TO .rda FILE.

  if (AnalysisType=="ClassComparison" | AnalysisType=="SAM" | (LikeGOComparison & !exists("StatusVariable"))) {
    if (!isPaired) {
      if (HasBlock) {
        experiments <- data.frame(ExpId=arrayIds, pheno=pheno, block=block)
      } else {
        experiments <- data.frame(ExpId=arrayIds, pheno=pheno)
      }
    } else {
      experiments <- data.frame(ExpId=arrayIds, pheno=pheno, id=id)
    }
  }
  
  if (AnalysisType=="BlockClassComparison") experiments <- data.frame(ExpId=id, pheno=pheno)
  if (AnalysisType=="Correlation")          experiments <- data.frame(ExpId=arrayIds, Correlation=pheno)
  if (AnalysisType=="Survival" | (LikeGOComparison & exists("StatusVariable")))  experiments <- data.frame(ExpId=arrayIds, phenoClass=phenoClass, phenoTime=phenoTime)
  
  save(experiments, file=paste(WorkPath, "/experiments.rda", sep=""))
  
  # geneIdNames.rda (originally created from Utilities.bas)
  GeneIdNames <- colnames(geneIds)
  save(GeneIdNames, file = file.path(WorkPath, "GeneIdNames.rda"))
  for(i in 1:length(GeneIdNames)) {
    assign(paste0("GeneId", i), geneIds[FilterAnalysisSet==1, i])
    save(list = paste0("GeneId", i), file = file.path(WorkPath, paste0("GeneId", i, ".rda")))
  }

# OUTPUT "exprData" TO "LR_data.txt" FILE.

  for (j in 1:ncol(exprData)) {
    isna <- is.na(exprData[,j])
    #note to use <-, keep in mind that treating exprData as global variable!
    if (any(isna)) {
      exprData[isna,j] <- -9999999
      gc()
    }  
  }
  
  if (DoRandomVariance) {
    #note to use <-, keep in mind that treating exprData as global variable!
    exprData <- cbind(exprData,FilterAnalysisSet)
    gc()   
  }  
  
  if (AnalysisType=="ClassComparison" | AnalysisType=="BlockClassComparison" | AnalysisType=="SAM" | 
      (LikeGOComparison & !exists("StatusVariable"))) {
    temp <- sum(pheno==ClassLevels[1])
    for (i in 2:NumClassLevels) {
      temp <- paste(temp, sum(pheno==ClassLevels[i]))
    }
    cat(paste(length(unique(ClassLevels)), nrow(exprData), ifelse(isPaired, 1, 0)), 0, "\n", 
        # the final '0' will be ignored for class comparison. See survival case where it was really used.
        file=paste(WorkPath, "/LR_data.txt", sep=""), append=FALSE)
    cat(temp, "\n",      
        file=paste(WorkPath, "/LR_data.txt", sep=""), append=TRUE)
    if (HasBlock) {
      cat(paste(as.double(as.factor(block)),collapse=" "),
          file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\n", append=TRUE)
    }
    if (AnalysisType=="SAM") save(exprData, pheno, ClassLevels, file=paste(WorkPath, "/SAM.rda", sep="")) 
  }
  
  if (AnalysisType == "Survival" | (LikeGOComparison & exists("StatusVariable"))) {
    # ArrayTools allows all status=1, but not allows all status=0.
    # If sum(n)=n(1), Fortran will recognize that all status=1.
    temp <- sum(phenoClass==ClassLevels[1])
    if (NumClassLevels > 1) {
      for (i in 2:NumClassLevels) temp <- paste(temp, sum(phenoClass==ClassLevels[i]))
    }  
    cat(length(unique(ClassLevels)),
        ifelse(LikeGOComparison, sum(FilterAnalysisSet==1), nrow(exprData)),
        ifelse(isPaired, 1, 0), event, "\n",   # Add 'event' to distinguish the case when all data are events/censored. 11/6/2013
        file=paste(WorkPath, "/LR_data.txt", sep=""), append=FALSE) # MLI 10/14/2009 fix a bug in Geneset + survival.
    cat(temp, "\n", file=paste(WorkPath, "/LR_data.txt", sep=""), append=TRUE) 
    write.table(paste(phenoTime,collapse=" "),file=paste(WorkPath, "/LR_data.txt", sep=""),
                sep="\t", quote=FALSE, row.names=FALSE, col.names=FALSE, append=TRUE)
  }
  
  if (AnalysisType == "Correlation") {
    cat(paste(length(pheno), nrow(exprData), ifelse(UseSpearmanTest, 2, 1)),
        file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\n", append=FALSE)
    write.table(paste(pheno,collapse=" "),file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\t", quote=FALSE, row.names=FALSE, col.names=FALSE, append=TRUE)
  }
  if (LikeGOComparison && exists("StatusVariable")) {
    # no RVM possibility, so just use filtered and probe reducted genes. It should match with the geneset_data.txt  MLI 10/14/2009
    write.table(exprData[FilterAnalysisSet==1,,drop=FALSE], file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\t", quote=FALSE, row.names=FALSE, col.names=FALSE, append=TRUE)
  } else {
    write.table(exprData, file=paste(WorkPath, "/LR_data.txt", sep=""), sep="\t", quote=FALSE, row.names=FALSE, col.names=FALSE, append=TRUE)
  }
  
# CREATE exprSet object for gene set comparison. Yong 4/18/07

  if (LikeGOComparison) {
    source.out<-try(source(paste(ArrayToolsPath,"/R/GenesetTests.R", sep=""), echo = FALSE)) 
    if (class(source.out)=='try-error') {
      require(tcltk)
      tkmessageBox(message=paste('Error occurred in sourcing GenesetTests.R:', geterrmessage(), sep='\n'),icon="info",type="ok")
    }
    
    if (DoRandomVariance) {
      arr.current <- exprData[, -ncol(exprData), drop = FALSE] #remove the last column FilterAnalysisSet when DoRandomVariance is True
    } else {
      arr.current <- exprData
    }
    arr.current <- arr.current[FilterAnalysisSet==1,,drop=FALSE] #now arr.current is the real filtered data matrix.
    if (!surv) myPheno <- data.frame(pheno) else myPheno <- data.frame(as.numeric(phenoClass), phenoTime)
    if (!CreateExprSetData(arr.current, myPheno, arrayIds, WorkPath)) {
      ReturnList$Msg <- "ERROR: exprSet object cannot be created for gene set comparison analysis."
      return(ReturnList)
    }
    # Added MLI 9/13/2009
    if (as.logical(DoGroupComparison)) {
      x <- arr.current
      y <- as.character(myPheno[,1])
      # check if the number of levels in both y and grp are 2.
      if (length(unique(grp)) != 2) {
        ReturnList$Msg <- "ERROR: the analysis works only for two groups data."
        return(ReturnList)
      }
      if (length(unique(y)) != 2 | any(table(y, grp)==1)) {
        #Qian 8/8/2012 Error message for current status, we may extend to multi-class.
        ReturnList$Msg <- "ERROR: only 2-class or paired data is allowed."
        return(ReturnList)
      }  
      gsanperm <- as.integer(gsanperm)  
      save(x, y, grp, arrayIds, DoRandomVariance, gsanperm,
           exprData, FilterAnalysisSet,
           isPaired, id,
           file=file.path(WorkPath, "GSEAgroupcomparison.rda"))
    }
  }
  
# Create file, class_perm_params.txt

# from ClassComparison.bas.
  
  #Write "/Fortran/class_perm_params.txt" file:
  #Deyun Modified for German/French language that turned decimal into comma. 4/24/06  ##
  classperm <- c(ifelse(DoExactPermTest, 1, 0), #1
                 NumPermMultivariate,           #2
                 ifelse(DoUnivariatePermTest, NumPermUnivariate, 0),#3
                 ifelse(DoUnivariateTest, 1, 0),#4
                 UnivarSignifLevel,             #5
                 ifelse(DoExpectedFDRN, 1, 0),  #6
                 ExpectedFDRN,                  #7
                 ifelse(DoExpectedFDRG, 1, 0),  #8
                 ExpectedFDRG,                  #9
                 ifelse(GlobalTest, 1, ifelse(DoUnivariateTest, 0, 1)), #10 do multivariate permutation
                 ifelse(DoUnivariateTest, 0.1, 1 - ConfLevel / 100),    #11 Ensure that Global p-value is used only with Univariate test. Supriya/Ming-Chung 07/18/07
                 ifelse(optNumber, MaxNumFD, 0),#12
                 MaxProportionFD,               #13
                 ifelse(DoRandomVariance, 1, 0),#14
                 0,                             #15
                 1,                             #16
                 ifelse(HasBlock, 1, 0),        #17
                 ifelse(DoFoldThresh, 1, 0),    #18
                 NumFoldThreshold,              #19
                 NumAlphaPairwise)              #20
  cat(classperm, file=file.path(WorkPath, "class_perm_params.txt"), sep = "\n")
  
  # DELETE ALL OBJECTS FROM RCOM MEMORY (EXCEPT "ReturnList" OJBECT
  
  rm(list=ls()[ls() != "ReturnList"])
  gc()
  
  return(ReturnList)
  
}  #End of function definition for WriteDataToFiles.
